@@ -1,8 +1,5 @@
 package Dancer2::Core::Request;
-{
-  $Dancer2::Core::Request::VERSION = '0.11';
-}
-
+$Dancer2::Core::Request::VERSION = '0.152000';
 # ABSTRACT: Interface for accessing incoming requests
 
 use Moo;
@@ -15,19 +12,10 @@ use URI::Escape;
 
 use Dancer2::Core::Types;
 use Dancer2::Core::Request::Upload;
+use Dancer2::Core::Cookie;
 
 with 'Dancer2::Core::Role::Headers';
 
-
-
-# check presence of XS module to speedup request
-eval { require URL::Encode::XS; };
-our $XS_URL_DECODE = !$@;
-
-eval { require CGI::Deurl::XS; };
-our $XS_PARSE_QUERY_STRING = !$@;
-
-
 # add an attribute for each HTTP_* variables
 # (HOST is managed manually)
 my @http_env_keys = (qw/
@@ -52,6 +40,12 @@ foreach my $attr ( @http_env_keys ) {
     );
 }
 
+# check presence of XS module to speedup request
+eval { require URL::Encode::XS; };
+our $XS_URL_DECODE = !$@;
+
+eval { require CGI::Deurl::XS; };
+our $XS_PARSE_QUERY_STRING = !$@;
 
 # then all the native attributes
 has env => (
@@ -60,7 +54,19 @@ has env => (
     default => sub { {} },
 );
 
+# a buffer for per-request variables
+has vars => (
+    is      => 'ro',
+    isa     => HashRef,
+    default => sub { {} },
+);
 
+sub var {
+    my $self = shift;
+    @_ == 2
+      ? $self->vars->{ $_[0] } = $_[1]
+      : $self->vars->{ $_[0] };
+}
 
 has path => (
     is      => 'ro',
@@ -92,9 +98,10 @@ sub _build_path {
 }
 
 has path_info => (
-    is      => 'rw',
+    is      => 'ro',
     isa     => Str,
     lazy    => 1,
+    writer  => 'set_path_info',
     builder => '_build_path_info',
 );
 
@@ -109,7 +116,6 @@ sub _build_path_info {
     return $self->path;
 }
 
-
 has method => (
     is      => 'rw',
     isa     => Dancer2HTTPMethod,
@@ -120,7 +126,6 @@ has method => (
     coerce => sub { uc $_[0] },
 );
 
-
 has content_type => (
     is      => 'ro',
     isa     => Str,
@@ -130,7 +135,6 @@ has content_type => (
     },
 );
 
-
 has content_length => (
     is      => 'ro',
     isa     => Num,
@@ -140,7 +144,6 @@ has content_length => (
     },
 );
 
-
 has body => (
     is      => 'ro',
     isa     => Str,
@@ -203,7 +206,6 @@ sub _set_route_params {
     $self->_build_params();
 }
 
-
 has uploads => (
     is  => 'ro',
     isa => HashRef,
@@ -216,8 +218,9 @@ has body_is_parsed => (
 );
 
 has is_behind_proxy => (
-    is      => 'rw',
+    is      => 'ro',
     isa     => Bool,
+    lazy    => 1,
     default => sub {0},
 );
 
@@ -225,24 +228,17 @@ sub host {
     my ($self) = @_;
 
     if ( $self->is_behind_proxy ) {
-        my @hosts = split /\s*,\s*/, (
-          $self->env->{HTTP_X_FORWARDED_HOST} || $self->env->{X_FORWARDED_HOST}
-        );
-
+        my @hosts = split /\s*,\s*/, $self->env->{HTTP_X_FORWARDED_HOST}, 2;
         return $hosts[0];
     } else {
         return $self->env->{'HTTP_HOST'};
     }
 }
 
-
-#from Dancer2::Core::Role::Headers
-
-
 # aliases, kept for backward compat
 sub agent                 { $_[0]->user_agent }
 sub remote_address        { $_[0]->address }
-sub forwarded_for_address { $_[0]->env->{'X_FORWARDED_FOR'} }
+sub forwarded_for_address { $_[0]->env->{HTTP_X_FORWARDED_FOR} }
 sub address               { $_[0]->env->{REMOTE_ADDR} }
 sub remote_host           { $_[0]->env->{REMOTE_HOST} }
 sub protocol              { $_[0]->env->{SERVER_PROTOCOL} }
@@ -251,15 +247,13 @@ sub request_uri           { $_[0]->env->{REQUEST_URI} }
 sub user                  { $_[0]->env->{REMOTE_USER} }
 sub script_name           { $_[0]->env->{SCRIPT_NAME} }
 
-
-
 sub scheme {
     my ($self) = @_;
     my $scheme;
     if ( $self->is_behind_proxy ) {
+        # Note the 'HTTP_' prefix the PSGI spec adds to headers.
         $scheme =
-             $self->env->{'X_FORWARDED_PROTOCOL'}
-          || $self->env->{'HTTP_X_FORWARDED_PROTOCOL'}
+             $self->env->{'HTTP_X_FORWARDED_PROTOCOL'}
           || $self->env->{'HTTP_X_FORWARDED_PROTO'}
           || $self->env->{'HTTP_FORWARDED_PROTO'}
           || "";
@@ -271,16 +265,12 @@ sub scheme {
       || "";
 }
 
-
 has serializer => (
     is        => 'ro',
     isa       => Maybe( ConsumerOf ['Dancer2::Core::Role::Serializer'] ),
     predicate => 1,
 );
 
-
-
-
 has data => (
     is      => 'ro',
     lazy    => 1,
@@ -300,8 +290,11 @@ sub deserialize {
 
     return unless $self->serializer->support_content_type($content_type);
 
+    # The latest draft of the RFC does not forbid DELETE to have content,
+    # rather the behaviour is undefined. Take the most lenient route and
+    # deserialize any content on delete as well.
     return
-      unless grep { $self->method eq $_ } qw/ PUT POST PATCH /;
+      unless grep { $self->method eq $_ } qw/ PUT POST PATCH DELETE /;
 
     # try to deserialize
     my $body = $self->_read_to_end();
@@ -318,7 +311,6 @@ sub deserialize {
     return $data;
 }
 
-
 sub secure    { $_[0]->scheme   eq 'https' }
 sub uri       { $_[0]->request_uri }
 sub is_head   { $_[0]->{method} eq 'HEAD' }
@@ -353,69 +345,11 @@ sub BUILD {
     $self->_build_uploads();
 }
 
-
 sub to_string {
     my ($self) = @_;
     return "[#" . $self->id . "] " . $self->method . " " . $self->path;
 }
 
-# Create a new request which is a clone of the current one, apart
-# from the path location, which points instead to the new location
-# TODO this could be written in a more clean manner with a clone mechanism
-sub make_forward_to {
-    my ( $self, $url, $params, $options ) = @_;
-
-    # we clone the env to make sure we don't alter the existing one in $self
-    my $env = { %{ $self->env } };
-
-    $env->{PATH_INFO} = $url;
-
-    my $new_request = ( ref $self )->new( env => $env, body_is_parsed => 1 );
-    my $new_params = _merge_params( scalar( $self->params ), $params || {} );
-
-    if ( exists( $options->{method} ) ) {
-        $new_request->method( $options->{method} );
-    }
-
-    # Copy params (these are already decoded)
-    $new_request->{_params}       = $new_params;
-    $new_request->{_body_params}  = $self->{_body_params};
-    $new_request->{_query_params} = $self->{_query_params};
-    $new_request->{_route_params} = $self->{_route_params};
-    $new_request->{body}          = $self->body;
-    $new_request->{headers}       = $self->headers;
-
-    return $new_request;
-}
-
-
-sub forward {
-    my ( $self, $context, $url, $params, $options ) = @_;
-    my $new_request = $self->make_forward_to( $url, $params, $options );
-
-    my $new_response = Dancer2->runner->server->dispatcher->dispatch(
-        $new_request->env,
-        $new_request,
-        $context,
-    );
-    # halt the response, so no further processing is done on this request.
-    # (any after hooks will have already been run)
-    $new_response->halt;
-    $context->response($new_response);
-    $context->with_return->($new_response) if $context->has_with_return;
-    return $new_response; # Should never be called..
-}
-
-sub _merge_params {
-    my ( $params, $to_add ) = @_;
-
-    for my $key ( keys %$to_add ) {
-        $params->{$key} = $to_add->{$key};
-    }
-    return $params;
-}
-
-
 sub base {
     my $self = shift;
     my $uri  = $self->_common_uri;
@@ -440,8 +374,6 @@ sub _common_uri {
     return $uri;
 }
 
-
-
 sub uri_base {
     my $self  = shift;
     my $uri   = $self->_common_uri;
@@ -454,7 +386,6 @@ sub uri_base {
     return $canon;
 }
 
-
 sub dispatch_path {
     my $self = shift;
 
@@ -474,7 +405,6 @@ sub dispatch_path {
     return $path;
 }
 
-
 sub uri_for {
     my ( $self, $part, $params, $dont_escape ) = @_;
 
@@ -491,8 +421,6 @@ sub uri_for {
     return $dont_escape ? uri_unescape( $uri->canonical ) : $uri->canonical;
 }
 
-
-
 sub params {
     my ( $self, $source ) = @_;
 
@@ -544,7 +472,6 @@ sub _decode {
     return $h;
 }
 
-
 sub is_ajax {
     my $self = shift;
 
@@ -554,7 +481,6 @@ sub is_ajax {
     return 1;
 }
 
-
 # context-aware accessor for uploads
 sub upload {
     my ( $self, $name ) = @_;
@@ -703,7 +629,7 @@ sub _init_request_headers {
                 ( my $field = $_ ) =~ s/^HTTPS?_//;
                 ( $field => $env->{$_} );
               }
-              grep {/^(?:HTTP|CONTENT|COOKIE)/i} keys %$env
+              grep {/^(?:HTTP|CONTENT)/i} keys %$env
         )
     );
 }
@@ -743,22 +669,21 @@ sub _build_uploads {
     $self->_build_params();
 }
 
-
 has cookies => (
-    is      => 'rw',
+    is      => 'ro',
     isa     => HashRef,
     lazy    => 1,
     builder => '_build_cookies',
 );
 
 sub _build_cookies {
-    my ($self) = @_;
-
-    my $env_str = $self->env->{COOKIE} || $self->env->{HTTP_COOKIE};
-    return {} unless defined $env_str;
-
+    my $self    = shift;
     my $cookies = {};
-    foreach my $cookie ( split( /[,;]\s/, $env_str ) ) {
+
+    my $http_cookie = $self->header('Cookie');
+    return $cookies unless defined $http_cookie; # nothing to do
+
+    foreach my $cookie ( split( /[,;]\s/, $http_cookie ) ) {
 
         # here, we don't want more than the 2 first elements
         # a cookie string can contains something like:
@@ -775,20 +700,21 @@ sub _build_cookies {
     return $cookies;
 }
 
-
 1;
 
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer2::Core::Request - Interface for accessing incoming requests
 
 =head1 VERSION
 
-version 0.11
+version 0.152000
 
 =head1 SYNOPSIS
 
@@ -814,7 +740,19 @@ a Dancer2 application.
 
 =head2 env()
 
-Return the current environment (C<%ENV>), as a hashref.
+Return the current PSGI environment hash reference.
+
+=head2 var
+
+By-name interface to variables stored in this request object.
+
+  my $stored = $request->var('some_variable');
+
+returns the value of 'some_variable', while
+
+  $request->var('some_variable' => 'value');
+
+will set it.
 
 =head2 path()
 
@@ -867,7 +805,7 @@ objects.
 
 It uses the environment hash table given to build the request object:
 
-    Dancer2::Core::Request->new(env => \%ENV);
+    Dancer2::Core::Request->new(env => \%env);
 
 It also accepts the C<body_is_parsed> boolean flag, if the new request object should
 not parse request body.
@@ -954,19 +892,6 @@ Alias to the PSGI input handle (C<< <request->env->{psgi.input}> >>)
 
 Return a string representing the request object (eg: C<"GET /some/path">)
 
-=head2 forward($request, $new_location)
-
-Create a new request which is a clone of the current one, apart
-from the path location, which points instead to the new location.
-This is used internally to chain requests using the forward keyword.
-
-Note that the new location should be a hash reference. Only one key is
-required, the C<to_url>, that should point to the URL that forward
-will use. Optional values are the key C<params> to a hash of
-parameters to be added to the current request parameters, and the key
-C<options> that points to a hash of options about the redirect (for
-instance, C<method> pointing to a new request method).
-
 =head2 base()
 
 Returns an absolute URI for the base of the application.  Returns a L<URI>
@@ -1064,11 +989,10 @@ C<uploads>.
 Returns a reference to a hash containing cookies, where the keys are the names of the
 cookies and values are L<Dancer2::Core::Cookie> objects.
 
-=head1 HTTP environment variables
+=head1 Common HTTP request headers
 
-All HTTP environment variables that are in %ENV will be provided in the
-L<Dancer2::Core::Request> object through specific accessors, here are those
-supported:
+Commonly used client-supplied HTTP request headers are available through
+specific accessors, here are those supported:
 
 =over 4
 
@@ -1108,6 +1032,13 @@ supported:
 
 =back
 
+With the exception of C<host>, these accessors are lookups into the PSGI env
+hash reference.
+
+Note that the L<PSGI> specification prefixes client-supplied request headers with
+C<HTTP_>. For example, a C<X-Requested-With> header has the key
+C<HTTP_X_REQUESTED_WITH> in the PSGI env hashref.
+
 =head1 EXTRA SPEED
 
 Install URL::Encode::XS and CGI::Deurl::XS for extra speed.
@@ -1124,7 +1055,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Alexis Sukrieh.
+This software is copyright (c) 2014 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
