@@ -1,19 +1,15 @@
-# ABSTRACT: Role for Handlers
-
 package Dancer2::Core::Role::Handler;
-{
-  $Dancer2::Core::Role::Handler::VERSION = '0.11';
-}
+# ABSTRACT: Role for Handlers
+$Dancer2::Core::Role::Handler::VERSION = '0.152000';
 use Moo::Role;
 use Dancer2::Core::Types;
 
-
 requires 'register';
 
-
 has app => (
     is  => 'ro',
     isa => InstanceOf ['Dancer2::Core::App'],
+    weak_ref => 1,
 );
 
 1;
@@ -22,13 +18,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer2::Core::Role::Handler - Role for Handlers
 
 =head1 VERSION
 
-version 0.11
+version 0.152000
 
 =head1 ATTRIBUTES
 
@@ -46,7 +44,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Alexis Sukrieh.
+This software is copyright (c) 2014 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
